/***************************************************************************
 * Copyright 2007      Jeremy Whiting <jpwhiting@kde.org>                  *
 * Copyright 2007      Frederik Gladhorn <frederik.gladhorn@kdemail.net>   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef KEDUVOCCOMMON_P_H
#define KEDUVOCCOMMON_P_H

#define KVD_VERS_PREFIX " v"       // kvoctrain v0.1.0

#endif
